package com.kakao.kakaolink.v2.network;

import android.content.Context;
import android.content.Intent;

import com.kakao.message.template.TemplateParams;
import com.kakao.network.IRequest;
import com.kakao.util.KakaoUtilService;

import org.json.JSONObject;

import java.util.Map;

/**
 * This is an interface that provides {@link IRequest} instance for using KakaoLink v2 validation API.
 * This also provides a helper that returns KakaoLink v2 intent.
 *
 * @author kevin.kang. Created on 2017. 11. 13..
 */

public interface KakaoLinkCore {
    /**
     * Checks whether KakaoLink v2 messages can be sent with the KakaoTalk installed on the device.
     *
     * @param context Context instance to be used in resolving intent
     * @return true if KakaoLink v2 can be sent with the installed KakaoTalk
     */
    boolean isAvailable(final Context context);

    /**
     *
     * Returns an {@link IRequest} object for default template validation API.
     *
     * @param context Current application context
     * @param appKey App key of the app that will be displayed in KakaoLink v2 message
     * @param params {@link TemplateParams}
     * @return {@link IRequest} instance
     */
    IRequest defaultTemplateRequest(final Context context, final String appKey, final TemplateParams params);

    /**
     * Returns an {@link IRequest} object for custom template validation API.
     *
     * @param context Current application context
     * @param appKey App key of the app that will be displayed in KakaoLink v2 message
     * @param templateId custom template id
     * @param templateArgs template arguments to be used in the custom template
     * @return {@link IRequest} object
     */
    IRequest customTemplateRequest(final Context context, final String appKey, final String templateId, final Map<String, String> templateArgs);

    /**
     *
     * Returns an {@link IRequest} object for simple scrap template validation API.
     *
     * @param context Current application context
     * @param appKey App key of the app that will be displayed in KakaoLink v2 message
     * @param url URL to be scrapped
     * @return {@link IRequest} instance
     */
    @SuppressWarnings("unused")
    IRequest scrapTemplateRequest(final Context context, final String appKey, final String url);

    /**
     * Returns an {@link IRequest} for scrap template validation API with custom template and
     * template arguments.
     *
     * @param context Current application context
     * @param url URL to be scrapped
     * @param templateId custom template id
     * @param templateArgs template arguments to be used in the custom template
     * @return {@link IRequest} instance
     */
    IRequest scrapTemplateRequest(final Context context, final String appKey, final String url, final String templateId, final Map<String, String> templateArgs);

    /**
     * Create an intent (kakaolink://send...) that will be used to send KakaoLink v2 message.
     *
     * @param context Current application context
     * @param appKey App key of the app that will be displayed in KakaoLink v2 message
     * @param linkResponse {@link JSONObject} response from validation API
     * @return {@link Intent} instance
     */
    Intent kakaoLinkIntent(final Context context, final String appKey, final JSONObject linkResponse);

    /**
     * Create a market install intent for KakaoTalk. This will also include referrer parameter
     * for the referrer application.
     *
     * @param context Current context
     * @return Market install intent for KakaoTalk
     */
    Intent kakaoTalkMarketIntent(final Context context);

    /**
     *
     */
    class Factory {
        /**
         * Retrieves  a singleton instance that sends KakaoLink v2 messages with the app key defined in
         * AndroidManifest.xml.
         *
         * @return IKakaoLinkSerice instance
         */
        public static KakaoLinkCore getInstance() {
            return service;
        }
        private static KakaoLinkCore service = new DefaultKakaoLinkCore(KakaoUtilService.Factory.getInstance());
    }
}
